@@ -46,7 +46,7 @@ Type itself.
 
 <h2>Implementation</h2>
 MatrixBase<ELT> uses MatrixHelper<S> for implementation, where S 
-is ELT::Scalar, that is, the underlying float, double, long double,
+is ELT::Scalar, that is, the underlying float, double,
 complex<float>, negator<conjugate<double>>, 
 etc. from which ELT is constructed. This is a finite set of which all
 members are explicitly instantiated in the implementation code, so 
@@ -100,7 +100,6 @@ public:
     typedef EPrecision Precision;     // complex removed from StdNumber
     typedef EScalarNormSq  ScalarNormSq;      // type of scalar^2
 
-    typedef MatrixBase<E>                T;
     typedef MatrixBase<ENeg>             TNeg;
     typedef MatrixBase<EWithoutNegator>  TWithoutNegator;
     typedef MatrixBase<EReal>            TReal;
@@ -404,10 +403,10 @@ public:
     template <class EE> inline MatrixBase& 
     colScaleInPlace(const VectorBase<EE>&);
 
-	template <class EE> inline void 
+    template <class EE> inline void 
     colScale(const VectorBase<EE>& c, typename EltResult<EE>::Mul& out) const;
 
-	template <class EE> inline typename EltResult<EE>::Mul
+    template <class EE> inline typename EltResult<EE>::Mul
     colScale(const VectorBase<EE>& c) const {
         typename EltResult<EE>::Mul out(nrow(), ncol()); colScale(c,out); return out;
     }
@@ -519,30 +518,30 @@ public:
         return out;
     }
 
-	/// M(i,j) *= R(i,j); R must have same dimensions as this.
-	template <class EE> inline MatrixBase& 
+    /// M(i,j) *= R(i,j); R must have same dimensions as this.
+    template <class EE> inline MatrixBase& 
     elementwiseMultiplyInPlace(const MatrixBase<EE>&);
 
-	template <class EE> inline void 
+    template <class EE> inline void 
     elementwiseMultiply(const MatrixBase<EE>&, typename EltResult<EE>::Mul&) const;
 
-	template <class EE> inline typename EltResult<EE>::Mul 
+    template <class EE> inline typename EltResult<EE>::Mul 
     elementwiseMultiply(const MatrixBase<EE>& m) const {
         typename EltResult<EE>::Mul out(nrow(), ncol()); 
         elementwiseMultiply<EE>(m,out); 
         return out;
     }
 
-	/// M(i,j) = R(i,j) * M(i,j); R must have same dimensions as this.
-	template <class EE> inline MatrixBase& 
+    /// M(i,j) = R(i,j) * M(i,j); R must have same dimensions as this.
+    template <class EE> inline MatrixBase& 
     elementwiseMultiplyFromLeftInPlace(const MatrixBase<EE>&);
 
-	template <class EE> inline void 
+    template <class EE> inline void 
     elementwiseMultiplyFromLeft(
         const MatrixBase<EE>&, 
         typename MatrixBase<EE>::template EltResult<E>::Mul&) const;
 
-	template <class EE> inline typename MatrixBase<EE>::template EltResult<E>::Mul 
+    template <class EE> inline typename MatrixBase<EE>::template EltResult<E>::Mul 
     elementwiseMultiplyFromLeft(const MatrixBase<EE>& m) const {
         typename EltResult<EE>::Mul out(nrow(), ncol()); 
         elementwiseMultiplyFromLeft<EE>(m,out); 
@@ -645,31 +644,6 @@ public:
         helper.dump(msg);
     }
 
-
-    // This routine is useful for implementing friendlier Matrix expressions and operators.
-    // It maps closely to the Level-3 BLAS family of pxxmm() routines like sgemm(). The
-    // operation performed assumes that "this" is the result, and that "this" has 
-    // already been sized correctly to receive the result. We'll compute
-    //     this = beta*this + alpha*A*B
-    // If beta is 0 then "this" can be uninitialized. If alpha is 0 we promise not
-    // to look at A or B. The routine can work efficiently even if A and/or B are transposed
-    // by their views, so an expression like
-    //        C += s * ~A * ~B
-    // can be performed with the single equivalent call
-    //        C.matmul(1., s, Tr(A), Tr(B))
-    // where Tr(A) indicates a transposed view of the original A.
-    // The ultimate efficiency of this call depends on the data layout and views which
-    // are used for the three matrices.
-    // NOTE: neither A nor B can be the same matrix as 'this', nor views of the same data
-    // which would expose elements of 'this' that will be modified by this operation.
-    template <class ELT_A, class ELT_B>
-    MatrixBase& matmul(const StdNumber& beta,   // applied to 'this'
-                       const StdNumber& alpha, const MatrixBase<ELT_A>& A, const MatrixBase<ELT_B>& B)
-    {
-        helper.matmul(beta,alpha,A.helper,B.helper);
-        return *this;
-    }
-
     /// Element selection for stored elements. These are the fastest element access
     /// methods but may not be able to access all elements of the logical matrix when
     /// some of its elements are not stored in memory. For example, a Hermitian matrix
@@ -894,6 +868,32 @@ private:
     MatrixHelper<Scalar> helper; // this is just one pointer
 
     template <class EE> friend class MatrixBase;
+
+    // ============================= Unimplemented =============================
+    // This routine is useful for implementing friendlier Matrix expressions and operators.
+    // It maps closely to the Level-3 BLAS family of pxxmm() routines like sgemm(). The
+    // operation performed assumes that "this" is the result, and that "this" has 
+    // already been sized correctly to receive the result. We'll compute
+    //     this = beta*this + alpha*A*B
+    // If beta is 0 then "this" can be uninitialized. If alpha is 0 we promise not
+    // to look at A or B. The routine can work efficiently even if A and/or B are transposed
+    // by their views, so an expression like
+    //        C += s * ~A * ~B
+    // can be performed with the single equivalent call
+    //        C.matmul(1., s, Tr(A), Tr(B))
+    // where Tr(A) indicates a transposed view of the original A.
+    // The ultimate efficiency of this call depends on the data layout and views which
+    // are used for the three matrices.
+    // NOTE: neither A nor B can be the same matrix as 'this', nor views of the same data
+    // which would expose elements of 'this' that will be modified by this operation.
+    template <class ELT_A, class ELT_B>
+    MatrixBase& matmul(const StdNumber& beta,   // applied to 'this'
+                       const StdNumber& alpha, const MatrixBase<ELT_A>& A, const MatrixBase<ELT_B>& B)
+    {
+        helper.matmul(beta,alpha,A.helper,B.helper);
+        return *this;
+    }
+
 };
 
 } //namespace SimTK
