@@ -195,7 +195,7 @@ private:
 
 
 //  ------------------------------ MatrixStorage -------------------------------
-/// Matrix "storage" refers to the physical layout of data in the computers 
+/// Matrix "storage" refers to the physical layout of data in the computer's
 /// memory. Whenever possible we attempt to store data in a format that enables 
 /// use of special high performance methods, such as those available in the 
 /// SimTK LAPACK/BLAS implementation.
@@ -505,7 +505,8 @@ public:
 
     /// This is an implicit conversion from the Condition enum to a
     /// MatrixCondition object.
-    MatrixCondition(Condition cond, Diagonal diag=UnknownDiagonal) : condition(cond) {}
+    MatrixCondition(Condition cond, Diagonal diag=UnknownDiagonal) 
+    :   condition(cond), diagonal(diag) {}
 
     /// Restore to default-constructed state of "none".
     MatrixCondition& setToNone() {condition=UnknownCondition; diagonal=UnknownDiagonal; return *this;}
@@ -518,7 +519,7 @@ public:
     static ConditionMask calcMask(Condition);
 
     /// Given a particular Diagonal condition provided as a commitment, calculate
-    /// the mask of all Diagonal conditions taht would satisfy that commitment.
+    /// the mask of all Diagonal conditions that would satisfy that commitment.
     /// For example, if the commitment is "RealDiagonal" then "ZeroDiagonal"
     /// and "OneDiagonal" would also be acceptable. If the Diagonal condition
     /// is specified as "UnknownDiagonal" then we'll return an Uncommitted mask.
@@ -902,7 +903,7 @@ public:
     MatrixCharacter::Mask::SizeMask getUpperBandwidthMask() const {return masks.uband;}
 
     int getDefaultNumRows() const {return masks.getDefaultNumRows();}
-    int getDefaultNumCols() const {return masks.getDefaultNumRows();}
+    int getDefaultNumCols() const {return masks.getDefaultNumCols();}
 
     bool isSizeOK(int m, int n) const {return masks.isSizeOK(m,n);} 
     bool isSizeOK(const std::pair<int,int>& mn) const
