@@ -234,6 +234,7 @@ public:
 
     /** These compile-time constants are required of every Composite
     Numerical Type (CNT). **/
+    #ifndef SWIG
     enum {
         NRows               = M,
         NCols               = 1,
@@ -255,6 +256,7 @@ public:
         IsPrecision         = 0,
         SignInterpretation  = CNT<E>::SignInterpretation
     };
+    #endif
 
     // These are reinterpretations of the current data, so have the
     // same packing (stride).
@@ -420,7 +422,7 @@ public:
     /** Default construction initializes %Vec's elements to NaN when debugging 
     but leaves them uninitialized garbage otherwise, so declarations have zero
     cost in Release builds. **/
-	Vec(){ 
+    Vec(){ 
     #ifndef NDEBUG
         setToNaN();
     #endif
@@ -591,7 +593,7 @@ public:
     /** Select an element of this %Vec and return a const reference to it.
     This is range-checked in Debug builds but has zero overhead in Release
     builds. **/
-	const E& operator[](int i) const 
+    const E& operator[](int i) const 
     {   assert(0 <= i && i < M); return d[i*STRIDE]; }
     /** Same as const operator[] above. **/
     const E& operator()(int i) const {return (*this)[i];}
@@ -599,9 +601,9 @@ public:
     /** Select an element of this %Vec and return a writable reference 
     to it. This is range-checked in Debug builds but has zero overhead in 
     Release builds. **/
-	E& operator[](int i) {assert(0 <= i && i < M); return d[i*STRIDE];}
+    E& operator[](int i) {assert(0 <= i && i < M); return d[i*STRIDE];}
     /** Same as non-const operator[] above. **/
-	E& operator()(int i) {return (*this)[i];}
+    E& operator()(int i) {return (*this)[i];}
 
     ScalarNormSq normSqr() const { return scalarNormSqr(); }
     typename CNT<ScalarNormSq>::TSqrt 
@@ -944,7 +946,7 @@ public:
         return true;
     }
 
-    /** For approximate comparisions, the default tolerance to use for a vector is
+    /** For approximate comparisons, the default tolerance to use for a vector is
     the same as its elements' default tolerance. **/
     static double getDefaultTolerance() {return CNT<ELT>::getDefaultTolerance();}
 
@@ -984,9 +986,9 @@ public:
     // Functions to be used for Scripting in MATLAB and languages that do not support operator overloading
     /** Print Vec into a string and return it.  Please refer to operator<< for details. **/
     std::string toString() const {
-		std::stringstream stream;
-		stream <<  (*this);
-		return stream.str(); 
+        std::stringstream stream;
+        stream <<  (*this);
+        return stream.str(); 
     }
 
     /** Variant of operator[] that's scripting friendly to set ith entry **/
@@ -1000,7 +1002,7 @@ public:
 private:
     // TODO: should be an array of scalars rather than elements to control
     // packing more carefully.
-	ELT d[NActualElements];    // data
+    ELT d[NActualElements];    // data
 };
 
 /////////////////////////////////////////////
@@ -1117,14 +1119,6 @@ template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<double>::Mul
 operator*(const double& l, const Vec<M,E,S>& r) {return r*l;}
 
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Mul
-operator*(const Vec<M,E,S>& l, const long double& r)
-  { return Vec<M,E,S>::template Result<long double>::MulOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Mul
-operator*(const long double& l, const Vec<M,E,S>& r) {return r*l;}
-
 // v = v*int, int*v -- just convert int to v's precision float
 template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<typename CNT<E>::Precision>::Mul
@@ -1184,15 +1178,6 @@ typename CNT<double>::template Result<Vec<M,E,S> >::Dvd
 operator/(const double& l, const Vec<M,E,S>& r)
   { return CNT<double>::template Result<Vec<M,E,S> >::DvdOp::perform(l,r); }
 
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Dvd
-operator/(const Vec<M,E,S>& l, const long double& r)
-  { return Vec<M,E,S>::template Result<long double>::DvdOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename CNT<long double>::template Result<Vec<M,E,S> >::Dvd
-operator/(const long double& l, const Vec<M,E,S>& r)
-  { return CNT<long double>::template Result<Vec<M,E,S> >::DvdOp::perform(l,r); }
-
 // v = v/int, int/v -- just convert int to v's precision float
 template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<typename CNT<E>::Precision>::Dvd
@@ -1254,14 +1239,6 @@ template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<double>::Add
 operator+(const double& l, const Vec<M,E,S>& r) {return r+l;}
 
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Add
-operator+(const Vec<M,E,S>& l, const long double& r)
-  { return Vec<M,E,S>::template Result<long double>::AddOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Add
-operator+(const long double& l, const Vec<M,E,S>& r) {return r+l;}
-
 // v = v+int, int+v -- just convert int to v's precision float
 template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<typename CNT<E>::Precision>::Add
@@ -1318,15 +1295,6 @@ typename CNT<double>::template Result<Vec<M,E,S> >::Sub
 operator-(const double& l, const Vec<M,E,S>& r)
   { return CNT<double>::template Result<Vec<M,E,S> >::SubOp::perform(l,r); }
 
-template <int M, class E, int S> inline
-typename Vec<M,E,S>::template Result<long double>::Sub
-operator-(const Vec<M,E,S>& l, const long double& r)
-  { return Vec<M,E,S>::template Result<long double>::SubOp::perform(l,r); }
-template <int M, class E, int S> inline
-typename CNT<long double>::template Result<Vec<M,E,S> >::Sub
-operator-(const long double& l, const Vec<M,E,S>& r)
-  { return CNT<long double>::template Result<Vec<M,E,S> >::SubOp::perform(l,r); }
-
 // v = v-int, int-v // just convert int to v's precision float
 template <int M, class E, int S> inline
 typename Vec<M,E,S>::template Result<typename CNT<E>::Precision>::Sub
