@@ -6,7 +6,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org/home/simbody.  *
  *                                                                            *
- * Portions copyright (c) 2010-12 Stanford University and the Authors.        *
+ * Portions copyright (c) 2010-15 Stanford University and the Authors.        *
  * Authors: Michael Sherman                                                   *
  * Contributors:                                                              *
  *                                                                            *
@@ -29,9 +29,12 @@
 #include "SimTKcommon/Testing.h"
 
 #include <vector>
+#include <set>
 #include <sstream>
 #include <iterator>
 #include <iostream>
+#include <utility>
+#include <memory>
 using std::cout;
 using std::endl;
 using std::cin;
@@ -110,6 +113,9 @@ private:
 namespace SimTK {
 template <> struct NiceTypeName<SmallIx> {
     static const char* name() {return "SmallIx";}
+    static const std::string& namestr() 
+    {   static const std::string ns(name()); return ns; }
+    static const std::string& xmlstr() {return namestr();}
 };
 }
 
@@ -182,30 +188,38 @@ template <class T> Counter Count<T>::initAssign;
 template <class T> Counter Count<T>::copyAssign;
 template <class T> Counter Count<T>::dtor;
 
+typedef std::set<float>::const_iterator inputIt; // not a random access iterator
+
 // Instantiate the whole class to check for compilation problems.
 namespace SimTK {
 template class Array_<int>;
 template class Array_<std::string, unsigned char>;
-};
 
-#ifdef _MSC_VER // gcc 4.1.2 had trouble with this
 // Instantiate templatized methods
-typedef std::set<float>::const_iterator inputIt; // not a random access iterator
 
 // Constructors.
-template Array_<float,int>::Array_(const Array_<float,int>&);
 template Array_<float,int>::Array_(const float*,const float*);
 
+
+// Gcc 4.8.2 complains that these two instantiations are ambiguous, and 
+// clang 3.4 dies with an internal error. cl has no problem with them. It is
+// hard to determine who is right so I'll just include this for cl for now.
+#ifdef _MSC_VER
+    template Array_<float,int>::Array_(const Array_<float,int>&);
+    template Array_<float,int>&
+    Array_<float,int>::operator=(const Array_<float,int>&);
+#endif
+
 // Assignment.
 template void 
 Array_<float,int>::assign(const float*,const float*);
 template void
 Array_<double,int>::assign(const inputIt&, const inputIt&);
-template Array_<float,int>& 
-Array_<float,int>::operator=(const Array_<float,int>&);
 template Array_<double,int>& 
 Array_<double,int>::operator=(const std::vector<float>&);
 
+
+
 // Insertion
 template float*
 Array_<float,int>::insert(float*, const float*, const float*);
@@ -213,10 +227,9 @@ template float*
 Array_<float,short>::insert(float*, const inputIt&, const inputIt&);
 
 // Comparison
-template bool SimTK::operator==(const ArrayViewConst_<float,int>&, 
-                                const ArrayViewConst_<float,unsigned>&);
-#endif
-
+template bool operator==(const ArrayViewConst_<float,int>&, 
+                         const ArrayViewConst_<float,unsigned>&);
+};
 
 void testConstruction() {
     const int data[] = {5,3,-2,27,9};
@@ -684,7 +697,6 @@ void testInputIterator() {
 
     std::istringstream fin8(" 9.1e2,9.2e2,9.2e3,");  fin8 >> fmid;
     SimTK_TEST(!fin8.fail()); // trailing comma OK here because we got our fill
-
 }
 
 // Reduce the loop count by 50X in Debug.
@@ -696,7 +708,8 @@ static const int Outer = 500000
 
 static const int Inner = 1000;
 void testSpeedStdVector() {
-    std::vector<int> v; 
+    std::vector<int> v;
+    using Index = std::vector<int>::size_type;
     v.reserve(Inner);
 
     for (int i=0; i < Outer; ++i) {
@@ -708,7 +721,7 @@ void testSpeedStdVector() {
     int sum;
     for (int i=0; i < Outer; ++i) {
         sum = i;
-        for (unsigned i=0; i < v.size(); ++i)
+        for (Index i=0; i < v.size(); ++i)
             sum += v[i];
     }
     cout << "std::vector sum=" << sum << endl;
@@ -716,6 +729,7 @@ void testSpeedStdVector() {
 
 void testSpeedSimTKArray() {
     Array_<int> v; 
+    using Index = Array_<int>::size_type;
     v.reserve(Inner);
 
     for (int i=0; i < Outer; ++i) {
@@ -727,7 +741,7 @@ void testSpeedSimTKArray() {
     int sum;
     for (int i=0; i < Outer; ++i) {
         sum = i;
-        for (unsigned i=0; i < v.size(); ++i)
+        for (Index i=0; i < v.size(); ++i)
             sum += v[i];
     }
     cout << "Array sum=" << sum << endl;
@@ -755,7 +769,14 @@ void testNiceTypeName() {
         << NiceTypeName<ArrayIndexPackType<long>::packed_size_type>::name() << endl;
     cout << "packed_size_type<unsigned long long>=" 
         << NiceTypeName<ArrayIndexPackType<unsigned long long>::packed_size_type>::name() << endl;
-    cout << NiceTypeName< Array_<String,char> >::name() << endl;
+    cout << "Array_<String,char> using name(): " 
+         << NiceTypeName< Array_<String,char> >::name() << endl;
+    // Check demangling/canonicalizing.
+    cout << "Array_<String,char> using namestr(): " 
+         << NiceTypeName< Array_<String,char> >::namestr() << endl;
+    // Check removing angle brackets for XML.
+    cout << "Array_<String,char> using xmlstr(): " 
+         << NiceTypeName< Array_<String,char> >::xmlstr() << endl;
 }
 
 // The Array_ class is supposed to make better use of memory than does
@@ -771,7 +792,7 @@ void testMemoryFootprint() {
     SimTK_TEST(sizeof(Array_<int,unsigned short>) <  sizeof(std::vector<int>));
 
     // Since an int is smaller than a pointer here we will do better than
-    // any 3-pointer implementation. And we shouldn't be worse then normal
+    // any 3-pointer implementation. And we shouldn't be worse than normal
     // for long longs.
     if (Is64BitPlatform) {
         SimTK_TEST(sizeof(Array_<int,int>)       <  sizeof(std::vector<int>));
@@ -847,10 +868,136 @@ void testMemoryFootprint() {
     }
 }
 
+// Create a local array and return it along with the original data location.
+// With move construction the caller should end up with our local data without
+// having to copy it.
+static std::pair<Array_<double,char>, double*> returnByValue(double d) {
+    Array_<double,char> local{1,2,3,4,5.5};
+    local.push_back(d);
+    double* localData = local.data();
+    return {std::move(local),localData};
+}
+
+void testMoveConstructionAndAssignment() {
+    Array_<double> ad1{1,2,3.5,4};
+    const double* p1 = ad1.data();
+    Array_<double> ad2{.01,.02};
+    const double* p2 = ad2.data();
+
+    Array_<double> ad3(ad1); // copy construction
+    const double* p3 = ad3.data();
+    SimTK_TEST(p3 != p2);
+    ad3 = std::move(ad1);    // move assignment
+    SimTK_TEST(ad3.data() == p1 && ad1.data() == p3);
+
+    Array_<double> ad4(std::move(ad2)); // move construction
+    SimTK_TEST(ad4.data()==p2 && ad2.empty());
+
+    auto returned = returnByValue(3.25); // construction
+    SimTK_TEST(returned.first == std::vector<double>({1,2,3,4,5.5,3.25}));
+    SimTK_TEST(returned.first.data() == returned.second);
+
+    returned = returnByValue(-1);       // assignment
+    SimTK_TEST(returned.first == std::vector<double>({1,2,3,4,5.5,-1}));
+    SimTK_TEST(returned.first.data() == returned.second);
+
+    // std::unique_ptr has only move construction so this won't compile if
+    // Array_ requires copy construction
+    Array_<std::unique_ptr<double>> aud;
+    aud.push_back(std::unique_ptr<double>(new double(5.125)));
+    aud.push_back(std::unique_ptr<double>(new double(3.5)));
+    aud.push_back(std::unique_ptr<double>(new double(-2.25)));
+    SimTK_TEST(aud.size()==3);
+    SimTK_TEST(*aud[0]==5.125 && *aud[1]==3.5 && *aud[2]==-2.25);
+
+    aud.emplace_back(new double(123.));
+    SimTK_TEST(aud.size()==4 && *aud[3]==123.);
+
+    aud.emplace(&aud[2], new double(100));
+    SimTK_TEST(aud.size()==5 && *aud[2]==100. && *aud[3]==-2.25);
+}
+
+template <class T>
+static void takeAnArray(const Array_<T>& arr) {
+}
+
+// Array_<T> has a non-explicit constructor that accepts an 
+// std::initializer_list<T> which should provide implicit conversion from an
+// initializer list to an Array_ and allow initializer_list<T2> as long as
+// T(T2) works (the compiler takes care of that while building the initializer
+// list).
+void testInitializerList() {
+    Array_<double> ad1{}; // Should call default constructor
+    SimTK_TEST(ad1.empty());
+    Array_<double> ad2{3}; // Should be 1-element initializer list
+    SimTK_TEST(ad2.size()==1 && ad2.front()==3);
+    Array_<double> ad3(3); // Should be a 3-element uninitialized list
+    SimTK_TEST(ad3.size()==3);
+
+    Array_<double> ad4 = {1,2,2.5,.125}; // initlist construction
+    SimTK_TEST(ad4 == std::vector<double>({1,2,2.5,.125}));
+    ad4 = {2,4,5};                   // implicit conversion, then move
+    SimTK_TEST(ad4 == std::vector<double>({2.,4.,5.}));
+    takeAnArray<int>({2,3,4}); // implicit conversion to Array_<int>
+    takeAnArray<double>({1.2,3,4}); // implicit conversion to Array_<double>
+}
+
+namespace {
+void argConversion(const Array_<TestIx>& arg) {
+    // try passing this a convertible type
+}
+void argConversion(const std::vector<TestIx>& arg) {
+    // try passing this a convertible type
+}
+
+// This is implicitly convertible to TextIx.
+class SubTestIx : public TestIx {
+public: explicit SubTestIx(int ix) : TestIx(ix) {}
+};
+}
+
+// Should be able to copy or assign arrays with different element types
+// provided that the source type is implicitly convertible to the 
+// destination type, but should fail even if there is an explicit
+// conversion.
+void testTypeMismatch() {
+    Array_<TestIx> txarray(2, TestIx(3));
+    Array_<SubTestIx> stxarray(2, SubTestIx(4));
+    Array_<SmallIx> sxarray(2, SmallIx(1));
+    TestIx tx; SmallIx sx;
+    // tx = sx;  // should fail to compile
+    tx = TestIx(sx); // allowed
+    txarray = stxarray; // should work
+    SimTK_TEST(txarray == Array_<TestIx>(2, TestIx(4)));
+    // txarray = sxarray; // should fail to compile
+
+    //argConversion(sxarray); // should fail to compile
+    argConversion(stxarray); // should work
+}
+
+// For comparison of Array_ with std::vector. 
+// std::vector is more strict.
+void testTypeMismatchStdVector() {
+    std::vector<TestIx> txarray(2,TestIx(3));
+    std::vector<SubTestIx> stxarray(2,SubTestIx(4));
+    std::vector<SmallIx> sxarray(2,SmallIx(1));
+    TestIx tx; SmallIx sx;
+    // tx = sx;  // should fail to compile
+    tx = TestIx(sx); // allowed
+    //txarray = stxarray; // <-- fails despite implicit conversion
+    // txarray = sxarray; // should fail to compile
+
+    //argConversion(sxarray); // should fail to compile
+    //argConversion(stxarray); //<-- fails despite implicit conversion
+}
+
 int main() {
 
     SimTK_START_TEST("TestArray");
 
+        SimTK_SUBTEST(testTypeMismatch);
+        SimTK_SUBTEST(testInitializerList);
+        SimTK_SUBTEST(testMoveConstructionAndAssignment);
         SimTK_SUBTEST(testInsert);
         SimTK_SUBTEST(testArrayViewAssignment);
         SimTK_SUBTEST(testInputIterator);
