@@ -6,7 +6,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org/home/simbody.  *
  *                                                                            *
- * Portions copyright (c) 2010-12 Stanford University and the Authors.        *
+ * Portions copyright (c) 2010-15 Stanford University and the Authors.        *
  * Authors: Peter Eastman, Michael Sherman                                    *
  * Contributors:                                                              *
  *                                                                            *
@@ -41,29 +41,61 @@ using namespace std;
     #include <io.h>
     #include <process.h>
     #define READ _read
+    #define WRITEFUNC _write
+    #define CLOSE _close
 #else
     #include <unistd.h>
     #define READ read
+    #define WRITEFUNC write
+    #define CLOSE close
+#endif
+
+#ifdef _MSC_VER
+#pragma warning(disable:4996) // don't warn about strcat, sprintf, etc.
 #endif
 
 // gcc 4.4.3 complains bitterly if you don't check the return
 // status from the write() system call. This avoids those 
 // warnings and maybe, someday, will catch an error.
 #define WRITE(pipeno, buf, len) \
-   {int status=write((pipeno), (buf), (len)); \
+   {int status=WRITEFUNC((pipeno), (buf), (len)); \
     SimTK_ERRCHK4_ALWAYS(status!=-1, "VisualizerProtocol",  \
     "An attempt to write() %d bytes to pipe %d failed with errno=%d (%s).", \
     (len),(pipeno),errno,strerror(errno));}
 
 static int inPipe;
 
-// Create a pipe, using the right call for this platform.
-static int createPipe(int pipeHandles[2]) {
+// Create the pipe going *from* simulator *to* visualizer, so only the
+// read end should be inherited by the visualizer.
+static int createPipeSim2Viz(int sim2viz[2]) {
     const int status =
 #ifdef _WIN32
-        _pipe(pipeHandles, 16384, _O_BINARY);
+        _pipe(sim2viz, 16384, _O_BINARY|_O_NOINHERIT);
+        if (status==0) {
+            const int readfd = _dup(sim2viz[0]); // copy is inheritable
+            _close(sim2viz[0]); // close the non-inheritable version
+            sim2viz[0] = readfd; // replace with inheritable copy
+        }
 #else
-        pipe(pipeHandles);
+        pipe(sim2viz);
+#endif
+    return status;
+}
+
+
+// Create the pipe going *to* simulator *from* visualizer, so only the
+// write end should be inherited by the visualizer.
+static int createPipeViz2Sim(int viz2sim[2]) {
+    const int status =
+#ifdef _WIN32
+        _pipe(viz2sim, 16384, _O_BINARY|_O_NOINHERIT);
+        if (status==0) {
+            const int writefd = _dup(viz2sim[1]); // copy is inheritable
+            _close(viz2sim[1]); // close the non-inheritable version
+            viz2sim[1] = writefd; // replace with inheritable copy
+        }
+#else
+        pipe(viz2sim);
 #endif
     return status;
 }
@@ -72,13 +104,15 @@ static int createPipe(int pipeHandles[2]) {
 // this platform. We take two executables to try in order,
 // and return after the first one succeeds. If neither works, we throw
 // an error that is hopefully helful.
-static void spawnViz(const Array_<String>& searchPath,
-                     const String& appName, int toSimPipe, int fromSimPipe)
+static void spawnViz(const Array_<String>& searchPath, const String& appName, 
+                     int sim2vizPipe[2], int viz2simPipe[2])
 {
     int status;
-    char vizPipeToSim[32], vizPipeFromSim[32];
-    sprintf(vizPipeToSim, "%d", toSimPipe);
-    sprintf(vizPipeFromSim, "%d", fromSimPipe);
+
+    // Pass pipe numbers as command line arguments to the visualizer.
+    char vizReadFromSim[32], vizWriteToSim[32];
+    sprintf(vizReadFromSim, "%d", sim2vizPipe[0]);
+    sprintf(vizWriteToSim, "%d", viz2simPipe[1]);
 
     String exePath; // search path + appName
 
@@ -87,25 +121,35 @@ static void spawnViz(const Array_<String>& searchPath,
     for (unsigned i=0; i < searchPath.size(); ++i) {
         exePath = searchPath[i] + appName;
         handle = _spawnl(P_NOWAIT, exePath.c_str(), appName.c_str(), 
-                         vizPipeToSim, vizPipeFromSim, (const char*)0);
-        if (handle != -1)
+                         vizReadFromSim, vizWriteToSim, (const char*)0);
+        if (handle != -1) {
+            // success: visualizer is running
+            _close(sim2vizPipe[0]); // read end (belongs to visualizer)
+            _close(viz2simPipe[1]); // write end (belongs to visualizer)
             break; // success!
+        }
     }
     status = (handle==-1) ? -1 : 0;
 #else
     const pid_t pid = fork();
     if (pid == 0) {
-        // child process
+        // child process (the visualizer)
+        close(sim2vizPipe[1]); // write end (belongs to simulator)
+        close(viz2simPipe[0]); // read end(belongs to simulator)
         for (unsigned i=0; i < searchPath.size(); ++i) {
             exePath = searchPath[i] + appName;
             status = execl(exePath.c_str(), appName.c_str(), 
-                           vizPipeToSim, vizPipeFromSim, (const char*)0); 
+                           vizReadFromSim, vizWriteToSim, (const char*)0); 
             // if we get here the execl() failed
         }
         // fall through -- we failed on every try
     } else {
-        // parent process
+        // parent process (the simulator)
         status = (pid==-1) ? -1 : 0;
+        if (status == 0) {
+            close(sim2vizPipe[0]); // read end (belongs to visualizer)
+            close(viz2simPipe[1]); // write end (belongs to visualizer)
+        }
     }
 #endif
 
@@ -122,25 +166,39 @@ static void spawnViz(const Array_<String>& searchPath,
     }
 }
 
+class ReadingInterrupted : public std::exception {};
+
+// This will hang until the expected number of bytes has been received.
+// Throws ReadingInterrupted if the srcPipe is closed.
 static void readDataFromPipe(int srcPipe, unsigned char* buffer, int bytes) {
     int totalRead = 0;
-    while (totalRead < bytes)
-        totalRead += READ(srcPipe, buffer+totalRead, bytes-totalRead);
+    while (totalRead < bytes) {
+        auto retval = READ(srcPipe, buffer + totalRead, bytes - totalRead);
+        SimTK_ERRCHK4_ALWAYS(retval!=-1, "VisualizerProtocol",
+            "An attempt to read() %d bytes from pipe %d failed with errno=%d (%s).", 
+            bytes - totalRead, srcPipe, errno, strerror(errno));
+        // The pipe was closed, perhaps because simbody-visualizer was closed,
+        // or shutdownGUI() or ~VisualizerProtocol() was called.
+        // Without this check, we end up in an infinite loop if the user closes
+        // simbody-visualizer.
+        if (retval == 0) throw ReadingInterrupted();
+
+        totalRead += retval;
+    }
 }
 
-static void readData(unsigned char* buffer, int bytes) 
+// Throws ReadingInterrupted if inPipe is closed.
+static void readData(unsigned char* buffer, int bytes)
 {
     readDataFromPipe(inPipe, buffer, bytes);
 }
 
-static void* listenForVisualizerEvents(void* arg) {
-    Visualizer& visualizer = *reinterpret_cast<Visualizer*>(arg);
+static void listenForVisualizerEvents(Visualizer& visualizer) {
     unsigned char buffer[256];
 
     try
   { while (true) {
         // Receive a user input event.
-
         readData(buffer, 1);
         switch (buffer[0]) {
         case KeyPressed: {
@@ -181,22 +239,32 @@ static void* listenForVisualizerEvents(void* arg) {
                 (unsigned)buffer[0]);
         }
     }
+  } catch (const ReadingInterrupted&) {
+        // We were told (by the main thread) to stop listening, or
+        // simbody-visualizer closed.
   } catch (const std::exception& e) {
         std::cout << "Visualizer listenerThread: unrecoverable error:\n";
         std::cout << e.what() << std::endl;
-        return (void*)1;
     }
-    return (void*)0;
 }
 
 VisualizerProtocol::VisualizerProtocol
    (Visualizer& visualizer, const Array_<String>& userSearchPath) 
 {
-    // Launch the GUI application. We'll first look for one in the same directory
-    // as the running executable; then if that doesn't work we'll look in the
-    // bin subdirectory of the SimTK installation.
+    // Launch the GUI application. We'll first look for one in the same
+    // directory as the running executable; then if that doesn't work we'll
+    // look in the bin subdirectory of the SimTK installation.
 
-    const char* GuiAppName = "simbody-visualizer";
+    String vizExecutableName;
+    if (Pathname::environmentVariableExists("SIMBODY_VISUALIZER_NAME")) {
+        vizExecutableName =
+            Pathname::getEnvironmentVariable("SIMBODY_VISUALIZER_NAME");
+    } else {
+        vizExecutableName = "simbody-visualizer";
+        #ifndef NDEBUG
+            vizExecutableName += "_d";
+        #endif
+    }
 
     Array_<String> actualSearchPath;
     // Always start with the current executable's directory.
@@ -212,11 +280,30 @@ VisualizerProtocol::VisualizerProtocol
     if (Pathname::environmentVariableExists("SIMBODY_HOME")) {
         const std::string e = Pathname::getAbsoluteDirectoryPathname(
                 Pathname::getEnvironmentVariable("SIMBODY_HOME"));
-        actualSearchPath.push_back(Pathname::addDirectoryOffset(e,"bin"));
+        actualSearchPath.push_back(Pathname::addDirectoryOffset(e,
+                    SIMBODY_VISUALIZER_REL_INSTALL_DIR));
     } else if (Pathname::environmentVariableExists("SimTK_INSTALL_DIR")) {
         const std::string e = Pathname::getAbsoluteDirectoryPathname(
             Pathname::getEnvironmentVariable("SimTK_INSTALL_DIR"));
-        actualSearchPath.push_back(Pathname::addDirectoryOffset(e,"bin"));
+        actualSearchPath.push_back(Pathname::addDirectoryOffset(e,
+                    SIMBODY_VISUALIZER_REL_INSTALL_DIR));
+    }
+
+    // Try using the location of SimTKsimbody combined with the path from the
+    // SimTKsimbody library to the simbody-visualizer install location (only
+    // available on non-Windows platforms). We must provide a function that
+    // resides in SimTKsimbody.
+    std::string SimTKsimbodyDir;
+    if (Pathname::getFunctionLibraryDirectory((void*)createPipeSim2Viz,
+                                              SimTKsimbodyDir)) {
+        // We have the path to SimTKsimbody; now we combine this with the path
+        // from SimTKsimbody to simbody-visualizer (this assumes the
+        // installation has not been reorganized from what CMake originally
+        // specified).
+        std::string absPathToVizDir =
+          Pathname::getAbsoluteDirectoryPathnameUsingSpecifiedWorkingDirectory(
+                  SimTKsimbodyDir, SIMBODY_PATH_FROM_LIBDIR_TO_VIZ_DIR);
+        actualSearchPath.push_back(absPathToVizDir);
     }
 
     // Try the build-time install location:
@@ -229,26 +316,26 @@ VisualizerProtocol::VisualizerProtocol
 
     actualSearchPath.push_back(
         Pathname::addDirectoryOffset(def,
-            Pathname::addDirectoryOffset("Simbody", "bin")));
+            Pathname::addDirectoryOffset("Simbody", SIMBODY_VISUALIZER_REL_INSTALL_DIR)));
     actualSearchPath.push_back(
         Pathname::addDirectoryOffset(def,
-            Pathname::addDirectoryOffset("SimTK", "bin")));
+            Pathname::addDirectoryOffset("SimTK", SIMBODY_VISUALIZER_REL_INSTALL_DIR)));
 
+    // Pipe[0] is the read end, Pipe[1] is the write end.
     int sim2vizPipe[2], viz2simPipe[2], status;
 
     // Create pipe pair for communication from simulator to visualizer.
-    status = createPipe(sim2vizPipe);
+    status = createPipeSim2Viz(sim2vizPipe);
     SimTK_ASSERT_ALWAYS(status != -1, "VisualizerProtocol: Failed to open pipe");
-    outPipe = sim2vizPipe[1];
+    outPipe = sim2vizPipe[1]; // write here to talk to visualizer
 
     // Create pipe pair for communication from visualizer to simulator.
-    status = createPipe(viz2simPipe);
+    status = createPipeViz2Sim(viz2simPipe);
     SimTK_ASSERT_ALWAYS(status != -1, "VisualizerProtocol: Failed to open pipe");
-    inPipe = viz2simPipe[0];
+    inPipe = viz2simPipe[0]; // read from here to receive from visualizer
 
     // Spawn the visualizer gui, trying local first then installed version.
-    spawnViz(actualSearchPath,
-             GuiAppName, sim2vizPipe[0], viz2simPipe[1]);
+    spawnViz(actualSearchPath, vizExecutableName, sim2vizPipe, viz2simPipe);
 
     // Before we do anything else, attempt to exchange handshake messages with
     // the visualizer. This will throw an exception if anything goes wrong.
@@ -256,10 +343,8 @@ VisualizerProtocol::VisualizerProtocol
     shakeHandsWithGUI(outPipe, inPipe);
 
     // Spawn the thread to listen for events.
-
-    pthread_mutex_init(&sceneLock, NULL);
-    pthread_t thread;
-    pthread_create(&thread, NULL, listenForVisualizerEvents, &visualizer);
+    eventListenerThread = std::thread(listenForVisualizerEvents,
+            std::ref(visualizer));
 }
 
 // This is executed on the main thread at GUI startup and thus does not
@@ -315,22 +400,60 @@ void VisualizerProtocol::shakeHandsWithGUI(int toGUIPipe, int fromGUIPipe) {
 
 void VisualizerProtocol::shutdownGUI() {
     // Don't wait for scene completion; kill GUI now.
+    
+    // We no longer need to listen for events from the GUI. Stop the listener
+    // thread before writing to the out pipe, because attempting to write to
+    // the pipe may throw an exception. Shutting down the listener thread was
+    // added to solve an issue with OpenSim MATLAB bindings, wherein MATLAB
+    // would use more and more CPU each time a Visualizer was created.
+    stopListeningIfNecessary();
+    
     char command = Shutdown;
     WRITE(outPipe, &command, 1);
 }
 
+VisualizerProtocol::~VisualizerProtocol() {
+    // If shutdownGUI() was not called, then the listener thread is still
+    // running and we should kill it.
+    stopListeningIfNecessary();
+    int retval = CLOSE(outPipe); // TODO(chrisdembia) is this necessary?
+    if (retval == -1) {
+        std::cout << "Warning in Simbody VisualizerProtocol: "
+            << "An attempt to close() pipe " << outPipe
+            << " failed with errno=" << errno << " (" << strerror(errno) << ")."
+            << std::endl;
+    }
+}
+
+void VisualizerProtocol::stopListeningIfNecessary() {
+    if (eventListenerThread.joinable()) {
+        // Shut down the listener thread cleanly. Tell the GUI to tell the
+        // simulator's listener thread to stop listening, which will allow the
+        // the (simulator's) listener thread to die. WRITE throws an exception
+        // if it can't write to the pipe, which most likely means the GUI has
+        // been closed. If the GUI is closed, the listener thread has
+        // finished (though is still joinable), so we can ignore the exception.
+        try {
+            WRITE(outPipe, &StopCommunication, 1);
+        } catch (...) {}
+        eventListenerThread.join();
+    }
+}
+
 void VisualizerProtocol::beginScene(Real time) {
-    pthread_mutex_lock(&sceneLock);
+    sceneLockBeginFinishScene.lock();
     char command = StartOfScene;
     WRITE(outPipe, &command, 1);
     float fTime = (float)time;
     WRITE(outPipe, &fTime, sizeof(float));
+    // The sceneMutex is NOT unlocked at the end of this scope
+    // (sceneLockBeginFinishScene is a member variable); see finishScene().
 }
 
 void VisualizerProtocol::finishScene() {
     char command = EndOfScene;
     WRITE(outPipe, &command, 1);
-    pthread_mutex_unlock(&sceneLock);
+    sceneLockBeginFinishScene.unlock();
 }
 
 void VisualizerProtocol::drawBox(const Transform& X_GB, const Vec3& scale, const Vec4& color, int representation) {
@@ -407,6 +530,10 @@ void VisualizerProtocol::drawPolygonalMesh(const PolygonalMesh& mesh, const Tran
                 faces.push_back((unsigned short) mesh.getFaceVertex(i, j+1));
                 faces.push_back((unsigned short) newIndex);
             }
+            // Close the face (thanks, Alexandra Zobova).
+            faces.push_back((unsigned short) mesh.getFaceVertex(i, numVert-1));
+            faces.push_back((unsigned short) mesh.getFaceVertex(i, 0));
+            faces.push_back((unsigned short) newIndex);
         }
     }
     SimTK_ERRCHK1_ALWAYS(vertices.size() <= 65535*3, 
@@ -425,14 +552,14 @@ void VisualizerProtocol::drawPolygonalMesh(const PolygonalMesh& mesh, const Tran
     
     meshes[impl] = (unsigned short)index;    // insert new mesh
     WRITE(outPipe, &DefineMesh, 1);
-    unsigned short numVertices = (unsigned)vertices.size()/3;
-    unsigned short numFaces = (unsigned)faces.size()/3;
+    unsigned short numVertices = (unsigned short)(vertices.size()/3);
+    unsigned short numFaces = (unsigned short)(faces.size()/3);
     WRITE(outPipe, &numVertices, sizeof(short));
     WRITE(outPipe, &numFaces, sizeof(short));
     WRITE(outPipe, &vertices[0], (unsigned)(vertices.size()*sizeof(float)));
     WRITE(outPipe, &faces[0], (unsigned)(faces.size()*sizeof(short)));
 
-    drawMesh(X_GM, scale, color, (short) representation, index, 0);
+    drawMesh(X_GM, scale, color, (short) representation, (unsigned short)index, 0);
 }
 
 void VisualizerProtocol::
@@ -485,24 +612,28 @@ drawLine(const Vec3& end1, const Vec3& end2, const Vec4& color, Real thickness)
 }
 
 void VisualizerProtocol::
-drawText(const Vec3& position, const Vec3& scale, const Vec4& color, 
+drawText(const Transform& X_GT, const Vec3& scale, const Vec4& color, 
          const string& string, bool faceCamera, bool isScreenText) {
     SimTK_ERRCHK1_ALWAYS(string.size() <= 256,
         "VisualizerProtocol::drawText()",
         "Can't display DecorativeText longer than 256 characters;"
         " received text of length %u.", (unsigned)string.size());
     WRITE(outPipe, &AddText, 1);
-    float buffer[9];
-    buffer[0] = (float) position[0];
-    buffer[1] = (float) position[1];
-    buffer[2] = (float) position[2];
-    buffer[3] = (float) scale[0];
-    buffer[4] = (float) scale[1];
-    buffer[5] = (float) scale[2];
-    buffer[6] = (float) color[0];
-    buffer[7] = (float) color[1];
-    buffer[8] = (float) color[2];
-    WRITE(outPipe, buffer, 9*sizeof(float));
+    float buffer[12];
+    const Vec3 rot = X_GT.R().convertRotationToBodyFixedXYZ();
+    buffer[0] = (float) rot[0];
+    buffer[1] = (float) rot[1];
+    buffer[2] = (float) rot[2];
+    buffer[3] = (float) X_GT.p()[0];
+    buffer[4] = (float) X_GT.p()[1];
+    buffer[5] = (float) X_GT.p()[2];
+    buffer[6] = (float) scale[0];
+    buffer[7] = (float) scale[1];
+    buffer[8] = (float) scale[2];
+    buffer[9] = (float) color[0];
+    buffer[10]= (float) color[1];
+    buffer[11]= (float) color[2];
+    WRITE(outPipe, buffer, 12*sizeof(float));
     short face = (short)faceCamera;
     WRITE(outPipe, &face, sizeof(short));
     short screen = (short)isScreenText;
@@ -516,7 +647,7 @@ void VisualizerProtocol::
 drawCoords(const Transform& X_GF, const Vec3& axisLengths, const Vec4& color) {
     WRITE(outPipe, &AddCoords, 1);
     float buffer[12];
-    Vec3 rot = X_GF.R().convertRotationToBodyFixedXYZ();
+    const Vec3 rot = X_GF.R().convertRotationToBodyFixedXYZ();
     buffer[0] = (float) rot[0];
     buffer[1] = (float) rot[1];
     buffer[2] = (float) rot[2];
@@ -534,27 +665,26 @@ drawCoords(const Transform& X_GF, const Vec3& axisLengths, const Vec4& color) {
 
 void VisualizerProtocol::
 addMenu(const String& title, int id, const Array_<pair<String, int> >& items) {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &DefineMenu, 1);
-    short titleLength = title.size();
+    short titleLength = (short)title.size();
     WRITE(outPipe, &titleLength, sizeof(short));
     WRITE(outPipe, title.c_str(), titleLength);
     WRITE(outPipe, &id, sizeof(int));
-    short numItems = items.size();
+    short numItems = (short)items.size();
     WRITE(outPipe, &numItems, sizeof(short));
     for (int i = 0; i < numItems; i++) {
         int buffer[] = {items[i].second, items[i].first.size()};
         WRITE(outPipe, buffer, 2*sizeof(int));
         WRITE(outPipe, items[i].first.c_str(), items[i].first.size());
     }
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::
 addSlider(const String& title, int id, Real minVal, Real maxVal, Real value) {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &DefineSlider, 1);
-    short titleLength = title.size();
+    short titleLength = (short)title.size();
     WRITE(outPipe, &titleLength, sizeof(short));
     WRITE(outPipe, title.c_str(), titleLength);
     WRITE(outPipe, &id, sizeof(int));
@@ -563,44 +693,39 @@ addSlider(const String& title, int id, Real minVal, Real maxVal, Real value) {
     buffer[1] = (float) maxVal;
     buffer[2] = (float) value;
     WRITE(outPipe, buffer, 3*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 
 void VisualizerProtocol::setSliderValue(int id, Real newValue) const {
     const float value = (float)newValue;
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetSliderValue, 1);
     WRITE(outPipe, &id, sizeof(int));
     WRITE(outPipe, &value, sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setSliderRange(int id, Real newMin, Real newMax) const {
     float buffer[2];
     buffer[0] = (float)newMin; buffer[1] = (float)newMax;
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetSliderRange, 1);
     WRITE(outPipe, &id, sizeof(int));
     WRITE(outPipe, buffer, 2*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setWindowTitle(const String& title) const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetWindowTitle, 1);
-    short titleLength = title.size();
+    short titleLength = (short)title.size();
     WRITE(outPipe, &titleLength, sizeof(short));
     WRITE(outPipe, title.c_str(), titleLength);
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setMaxFrameRate(Real rate) const {
     const float frameRate = (float)rate;
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetMaxFrameRate, 1);
     WRITE(outPipe, &frameRate, sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 
@@ -609,54 +734,48 @@ void VisualizerProtocol::setBackgroundColor(const Vec3& color) const {
     buffer[0] = (float)color[0]; 
     buffer[1] = (float)color[1]; 
     buffer[2] = (float)color[2];
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetBackgroundColor, 1);
     WRITE(outPipe, buffer, 3*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setShowShadows(bool shouldShow) const {
     const short show = (short)shouldShow; // 0 or 1
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetShowShadows, 1);
     WRITE(outPipe, &show, sizeof(short));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setShowFrameRate(bool shouldShow) const {
     const short show = (short)shouldShow; // 0 or 1
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetShowFrameRate, 1);
     WRITE(outPipe, &show, sizeof(short));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setShowSimTime(bool shouldShow) const {
     const short show = (short)shouldShow; // 0 or 1
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetShowSimTime, 1);
     WRITE(outPipe, &show, sizeof(short));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setShowFrameNumber(bool shouldShow) const {
     const short show = (short)shouldShow; // 0 or 1
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetShowFrameNumber, 1);
     WRITE(outPipe, &show, sizeof(short));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setBackgroundType(Visualizer::BackgroundType type) const {
     const short backgroundType = (short)type;
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetBackgroundType, 1);
     WRITE(outPipe, &backgroundType, sizeof(short));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setCameraTransform(const Transform& X_GC) const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetCamera, 1);
     float buffer[6];
     Vec3 rot = X_GC.R().convertRotationToBodyFixedXYZ();
@@ -667,17 +786,15 @@ void VisualizerProtocol::setCameraTransform(const Transform& X_GC) const {
     buffer[4] = (float) X_GC.p()[1];
     buffer[5] = (float) X_GC.p()[2];
     WRITE(outPipe, buffer, 6*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::zoomCamera() const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &ZoomCamera, 1);
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::lookAt(const Vec3& point, const Vec3& upDirection) const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &LookAt, 1);
     float buffer[6];
     buffer[0] = (float) point[0];
@@ -687,45 +804,40 @@ void VisualizerProtocol::lookAt(const Vec3& point, const Vec3& upDirection) cons
     buffer[4] = (float) upDirection[1];
     buffer[5] = (float) upDirection[2];
     WRITE(outPipe, buffer, 6*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setFieldOfView(Real fov) const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetFieldOfView, 1);
     float buffer[1];
     buffer[0] = (float)fov;
     WRITE(outPipe, buffer, sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setClippingPlanes(Real near, Real far) const {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetClipPlanes, 1);
     float buffer[2];
     buffer[0] = (float)near;
     buffer[1] = (float)far;
     WRITE(outPipe, buffer, 2*sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::
 setSystemUpDirection(const CoordinateDirection& upDir) {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetSystemUpDirection, 1);
     const unsigned char axis = (unsigned char)upDir.getAxis();
     const signed char   sign = (signed char)upDir.getDirection();
     WRITE(outPipe, &axis, 1);
     WRITE(outPipe, &sign, 1);
-    pthread_mutex_unlock(&sceneLock);
 }
 
 void VisualizerProtocol::setGroundHeight(Real height) {
-    pthread_mutex_lock(&sceneLock);
+    std::lock_guard<std::mutex> lock(sceneMutex);
     WRITE(outPipe, &SetGroundHeight, 1);
     float heightBuffer = (float) height;
     WRITE(outPipe, &heightBuffer, sizeof(float));
-    pthread_mutex_unlock(&sceneLock);
 }
 
 
