@@ -37,7 +37,7 @@ State variables and computation results are organized into stages:
    Stage::Time
    Stage::Position  
    Stage::Velocity  
-   Stage::Dynamics      dynamic operators available
+   Stage::Dynamics      calculate forces
    Stage::Acceleration  response to forces in the state
   ---------------------------------------------------------
    Stage::Report        only used when outputting something
@@ -64,6 +64,7 @@ Whatever so that it can calculate results and put them in the cache (which is
 allocated if necessary), and then advance to stage Whatever. */
 
 #include "simbody/internal/common.h"
+#include "simbody/internal/Motion.h"
 
 #include <cassert>
 #include <iostream>
@@ -147,7 +148,16 @@ public:
     int sumSqDOFs;
 
     DiscreteVariableIndex modelingVarsIndex;
-    CacheEntryIndex       modelingCacheIndex;
+    CacheEntryIndex       modelingCacheIndex,instanceCacheIndex, timeCacheIndex, 
+                          treePositionCacheIndex, constrainedPositionCacheIndex,
+                          compositeBodyInertiaCacheIndex, 
+                          articulatedBodyInertiaCacheIndex,
+                          treeVelocityCacheIndex, constrainedVelocityCacheIndex,
+                          articulatedBodyVelocityCacheIndex,
+                          dynamicsCacheIndex, 
+                          treeAccelerationCacheIndex, 
+                          constrainedAccelerationCacheIndex;
+
 
     // These are instance variables that exist regardless of modeling
     // settings; they are instance variables corresponding to topological
@@ -254,12 +264,6 @@ public:
     UIndex uIndex;
     DiscreteVariableIndex timeVarsIndex, qVarsIndex, uVarsIndex, 
                           dynamicsVarsIndex, accelerationVarsIndex;
-    CacheEntryIndex       instanceCacheIndex, timeCacheIndex, 
-                          treePositionCacheIndex, constrainedPositionCacheIndex,
-                          compositeBodyInertiaCacheIndex, articulatedBodyInertiaCacheIndex,
-                          treeVelocityCacheIndex, constrainedVelocityCacheIndex,
-                          dynamicsCacheIndex, 
-                          treeAccelerationCacheIndex, constrainedAccelerationCacheIndex;
 
 private:
     // MobilizedBody 0 is Ground.
@@ -655,8 +659,6 @@ public:
     Transform&       updX_AB(AncestorConstrainedBodyPoolIndex cbpx)
     {   return constrainedBodyConfigInAncestor[cbpx]; }
 public:
-    // qerr cache space is provided directly by the State
-
     // At model stage, each mobilizer (RBNode) is given a chance to grab
     // a segment of this cache entry for its own private use. This includes
     // pre-calculated sincos(q) for mobilizers with angular coordinates,
@@ -821,49 +823,52 @@ public:
 // =============================================================================
 //                       ARTICULATED BODY INERTIA CACHE
 // =============================================================================
-// These articulated body inertias take into account prescribed motion, 
-// meaning that they are produced by a combination of articulated and rigid
-// shift operations depending on each mobilizer's current stats as "regular"
-// or "prescribed". That means that the articulated inertias here are suited
-// only for "mixed" dynamics; you can't use them to calculate M^-1*f unless
-// there is no prescribed motion in the system.
-//
-// Each articulated body inertia here is expressed in the Ground frame but 
-// measured about its body's origin.
-//
-// Articulated body inertia matrices, though symmetric and positive
-// definite, do not have the same simple structure as rigid-body (or composite-
-// body) spatial inertias. For example, the apparent mass depends on direction.
-// All 21 elements of this symmetric 6x6 matrix are unique, while there are only
-// 10 unique elements in a rigid body spatial inertia.
-//
-// Note that although we use some *rigid* body shift operations here, the 
-// results in general are all *articulated* body inertias, because a rigid shift 
-// of an articulated body inertia is still an articulated body inertia. Only if 
-// all mobilizers are prescribed will these be rigid body spatial inertias. For a 
-// discussion of the properties of articulated body inertias, see Section 7.1 
-// (pp. 119-123) of Roy Featherstone's excellent 2008 book, Rigid Body Dynamics 
-// Algorithms. 
-//
-// Intermediate quantities PPlus, D, DI, and G are calculated here
-// which are separately useful when dealing with "regular" mobilized bodies. 
-// These quantities are not calculated for prescribed mobilizers; they will 
-// remain NaN in that case. In
-// particular, this means that the prescribed-mobilizer mass properties do not 
-// have to be invertible, so you can have terminal massless bodies as long as 
-// their motion is always prescribed.
-// TODO: should D still be calculated? It doesn't require inversion.
-//
-// Articulated body inertias depend only on positions but are not usually needed 
-// until Acceleration stage. Thus this cache entry should have dependsOn stage 
-// Position, and computedBy stage Dynamics. However, it can be realized any
-// time after Position.
+/* These articulated body inertias (ABIs) take into account prescribed motion, 
+meaning that they are produced by a combination of articulated and rigid
+shift operations depending on each mobilizer's current status as "free"
+or "prescribed". That means that the articulated inertias here are suited
+only for "mixed" dynamics; you can't use them to calculate M^-1*f unless
+there is no prescribed motion in the system.
 
+Each articulated body inertia here is expressed in the Ground frame but 
+measured about its body's origin.
+
+Articulated body inertia matrices, though symmetric and positive
+definite, do not have the same simple structure as rigid-body (or composite-
+body) spatial inertias. For example, the apparent mass depends on direction.
+All 21 elements of this symmetric 6x6 matrix are unique, while there are only
+10 unique elements in a rigid body spatial inertia.
+
+Note that although we use some *rigid* body shift operations here, the 
+results in general are all *articulated* body inertias, because a rigid shift 
+of an articulated body inertia is still an articulated body inertia. Only if 
+all mobilizers are prescribed will these be rigid body spatial inertias. For 
+a discussion of the properties of articulated body inertias, see Section 7.1 
+(pp. 119-123) of Roy Featherstone's excellent 2008 book, Rigid Body Dynamics 
+Algorithms. 
+
+Intermediate quantities PPlus, D, DI, and G are calculated here which are 
+separately useful when dealing with "free" mobilized bodies. These quantities
+are not calculated for prescribed mobilizers; they will remain NaN in that 
+case. In particular, this means that the prescribed-mobilizer mass properties
+do not have to be invertible, so you can have terminal massless bodies as 
+long as their motion is always prescribed.
+TODO: should D still be calculated? It doesn't require inversion.
+
+ABIs depend only on position kinematics (they are time-independent) but are not
+usually needed until Acceleration stage, and we do not want to compute them
+earlier because (a) they are expensive, and (b) they require good, invertible
+mass properties. Some modelers don't bother with good mass properties since they
+have no intention of doing forward dynamics; they will rightfully complain
+if they get a singular mass matrix exception in Dynamics stage. So this cache
+entry should have depends-on stage Instance, with PositionKinematics as an 
+additional prerequisite, and computed-by stage Acceleration. However, it can
+be realized explicitly any time its stage and prerequisite are valid, such as
+after Stage::Position. */
 class SBArticulatedBodyInertiaCache {
 public:
-    Array_<ArticulatedInertia,MobilizedBodyIndex> articulatedBodyInertia; // nb (P)
-
-    Array_<ArticulatedInertia,MobilizedBodyIndex> pPlus; // nb
+    Array_<ArticulatedInertia,MobodIndex> articulatedBodyInertia; // nb (P)
+    Array_<ArticulatedInertia,MobodIndex> pPlus;                  // nb (PPlus)
 
     Vector_<Real>       storageForD;    // sum(nu[j]^2)
     Vector_<Real>       storageForDI;   // sum(nu[j]^2)
@@ -915,12 +920,18 @@ public:
 
 class SBTreeVelocityCache {
 public:
-    const SpatialVec& getV_FM(MobilizedBodyIndex mbx) const {return mobilizerRelativeVelocity[mbx];}
-    SpatialVec&       updV_FM(MobilizedBodyIndex mbx)       {return mobilizerRelativeVelocity[mbx];}
-    const SpatialVec& getV_PB(MobilizedBodyIndex mbx) const {return bodyVelocityInParent[mbx];}
-    SpatialVec&       updV_PB(MobilizedBodyIndex mbx)       {return bodyVelocityInParent[mbx];}
-    const SpatialVec& getV_GB(MobilizedBodyIndex mbx) const {return bodyVelocityInGround[mbx];}
-    SpatialVec&       updV_GB(MobilizedBodyIndex mbx)       {return bodyVelocityInGround[mbx];}
+    const SpatialVec& getV_FM(MobodIndex mbx) const 
+    {   return mobilizerRelativeVelocity[mbx]; }
+    SpatialVec&       updV_FM(MobodIndex mbx)       
+    {   return mobilizerRelativeVelocity[mbx]; }
+    const SpatialVec& getV_PB(MobodIndex mbx) const 
+    {   return bodyVelocityInParent[mbx]; }
+    SpatialVec&       updV_PB(MobodIndex mbx)       
+    {   return bodyVelocityInParent[mbx]; }
+    const SpatialVec& getV_GB(MobodIndex mbx) const 
+    {   return bodyVelocityInGround[mbx]; }
+    SpatialVec&       updV_GB(MobodIndex mbx)       
+    {   return bodyVelocityInGround[mbx]; }
 
     const SpatialVec& getV_AB(AncestorConstrainedBodyPoolIndex cbpx) const 
     {   return constrainedBodyVelocityInAncestor[cbpx]; }
@@ -930,9 +941,9 @@ public:
 public:
     // qdot cache space is supplied directly by the State
 
-    Array_<SpatialVec,MobilizedBodyIndex> mobilizerRelativeVelocity; // nb (V_FM) cross-mobilizer velocity
-    Array_<SpatialVec,MobilizedBodyIndex> bodyVelocityInParent;      // nb (V_PB)
-    Array_<SpatialVec,MobilizedBodyIndex> bodyVelocityInGround;      // nb (V_GB)
+    Array_<SpatialVec,MobodIndex> mobilizerRelativeVelocity; // nb (V_FM)
+    Array_<SpatialVec,MobodIndex> bodyVelocityInParent;      // nb (V_PB)
+    Array_<SpatialVec,MobodIndex> bodyVelocityInGround;      // nb (V_GB)
 
     // CAUTION: our definition of the H matrix is transposed from those used
     // by Jain and by Schwieters.
@@ -940,18 +951,20 @@ public:
     Array_<Vec3> storageForHDot;     // 2 x ndof (HDot_PB_G)
 
     // nb (VB_PB_G=HDot_PB_G*u)
-    Array_<SpatialVec,MobilizedBodyIndex> bodyVelocityInParentDerivRemainder; 
+    Array_<SpatialVec,MobodIndex> bodyVelocityInParentDerivRemainder; 
     
-    Array_<SpatialVec,MobilizedBodyIndex> gyroscopicForces;                // nb (b)
-    Array_<SpatialVec,MobilizedBodyIndex> mobilizerCoriolisAcceleration;   // nb (a)
-    Array_<SpatialVec,MobilizedBodyIndex> totalCoriolisAcceleration;       // nb (A)
+    Array_<SpatialVec,MobodIndex> gyroscopicForces;                // nb (b)
+    Array_<SpatialVec,MobodIndex> mobilizerCoriolisAcceleration;   // nb (a)
+    Array_<SpatialVec,MobodIndex> totalCoriolisAcceleration;       // nb (A)
+    Array_<SpatialVec,MobodIndex> totalCentrifugalForces;          // nb (M*A+b)
+
 
         // Ancestor Constrained Body Pool
 
-    // For Constraints whose Ancestor body A is not Ground G, we assign pool entries
-    // for each of their Constrained Bodies (call the total number 'nacb')
-    // to store the above information but measured and expressed in the Ancestor frame
-    // rather than Ground.
+    // For Constraints whose Ancestor body A is not Ground G, we assign pool 
+    // entries for each of their Constrained Bodies (call the total number 
+    // 'nacb') to store the above information but measured and expressed in the
+    // Ancestor frame rather than Ground.
     Array_<SpatialVec> constrainedBodyVelocityInAncestor; // nacb (V_AB)
 
 public:
@@ -961,33 +974,38 @@ public:
     {
         // Pull out construction-stage information from the tree.
         const int nBodies = tree.nBodies;
-        const int nDofs   = tree.nDOFs;     // this is the number of u's (nu)
-        const int maxNQs  = tree.maxNQs;  // allocate the max # q's we'll ever need
+        const int nDofs   = tree.nDOFs;  // this is the number of u's (nu)
+        const int maxNQs  = tree.maxNQs; // allocate max # q's we'll ever need
         const int nacb    = tree.nAncestorConstrainedBodies;
 
+        const SpatialVec SVZero(Vec3(0),Vec3(0));
+
         mobilizerRelativeVelocity.resize(nBodies);       
-        mobilizerRelativeVelocity[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        mobilizerRelativeVelocity[GroundIndex] = SVZero;
 
         bodyVelocityInParent.resize(nBodies);       
-        bodyVelocityInParent[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        bodyVelocityInParent[GroundIndex] = SVZero;
 
         bodyVelocityInGround.resize(nBodies);       
-        bodyVelocityInGround[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        bodyVelocityInGround[GroundIndex] = SVZero;
 
         storageForHDot_FM.resize(2*nDofs);
         storageForHDot.resize(2*nDofs);
 
         bodyVelocityInParentDerivRemainder.resize(nBodies);       
-        bodyVelocityInParentDerivRemainder[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        bodyVelocityInParentDerivRemainder[GroundIndex] = SVZero;
         
         gyroscopicForces.resize(nBodies);           
-        gyroscopicForces[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        gyroscopicForces[GroundIndex] = SVZero;
      
         mobilizerCoriolisAcceleration.resize(nBodies);       
-        mobilizerCoriolisAcceleration[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        mobilizerCoriolisAcceleration[GroundIndex] = SVZero;
 
         totalCoriolisAcceleration.resize(nBodies);       
-        totalCoriolisAcceleration[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
+        totalCoriolisAcceleration[GroundIndex] = SVZero;
+    
+        totalCentrifugalForces.resize(nBodies);           
+        totalCentrifugalForces[GroundIndex] = SVZero;
 
         constrainedBodyVelocityInAncestor.resize(nacb);
     }
@@ -1027,6 +1045,44 @@ public:
 //........................ CONSTRAINED VELOCITY CACHE ..........................
 
 
+// =============================================================================
+//                     ARTICULATED BODY VELOCITY CACHE
+// =============================================================================
+/* This cache entry holds any time-independent precalculations that are 
+dependent both on velocity kinematics and articulated body inertias (ABIs). 
+Currently that is just the articulated body centrifugal forces F=P*a+b where P 
+is the ABI, a is the cross-mobilizer Coriolis acceleration, and b is the
+gyroscopic force due to rotational inertia.
+
+We want to pre-calculate this term for use during Stage::Acceleration because
+it does not change as a result of changes to forces in Stage::Dynamics, and it
+is common to recalculate accelerations with only forces changing, especially
+when dealing with cosntrained systems. But we don't want ot calculate them
+prior to Stage::Acceleration because of the dependence on ABIs; see the 
+comments for SBArticulatedBodyInertiaCache for why. So this cache
+entry should have depends-on stage Instance, with VelocityKinematics and ABIs
+as an additional prerequisites, and computed-by stage Acceleration. However, it 
+can be realized any time its stage and prerequisites are valid, such 
+as after Stage::Velocity plus realizeArticulatedBodyInertias(). */
+class SBArticulatedBodyVelocityCache {
+public:
+    Array_<SpatialVec,MobodIndex> articulatedBodyCentrifugalForces; //nb (P*a+b)
+
+public:
+    void allocate(const SBTopologyCache& tree,
+                  const SBModelCache&,
+                  const SBInstanceCache&) 
+    {
+        // Pull out construction-stage information from the tree.
+        const int nBodies = tree.nBodies;
+        
+        articulatedBodyCentrifugalForces.resize(nBodies);           
+        articulatedBodyCentrifugalForces[GroundIndex] = 
+                                                    SpatialVec(Vec3(0),Vec3(0));
+    }
+};
+//...................... ARTICULATED BODY VELOCITY CACHE .......................
+
 
 // =============================================================================
 //                                DYNAMICS CACHE
@@ -1034,38 +1090,22 @@ public:
 class SBDynamicsCache {
 public:
     // This holds the values from all the Motion prescribed acceleration 
-    // calculations, and those which result from diffentiating prescribed velocities,
-    // or differentiating twice prescribed positions.
+    // calculations, and those which result from diffentiating prescribed 
+    // velocities, or twice-differentiating prescribed positions.
     Array_<Real> presUDotPool;    // Index with PresUDotPoolIndex
 
-    // Dynamics
-    // Here a=body's incremental contribution to coriolis acceleration
-    //      A=total coriolis acceleration for this body
-    //      b=gyroscopic force
-    Array_<SpatialVec,MobilizedBodyIndex> mobilizerCentrifugalForces; // nb (P*a+b)
-    Array_<SpatialVec,MobilizedBodyIndex> totalCentrifugalForces;     // nb (P*A+b)
-
     Array_<SpatialMat,MobilizedBodyIndex> Y;                          // nb
 
 public:
     void allocate(const SBTopologyCache& tree,
-                  const SBModelCache&    model,
+                  const SBModelCache&,
                   const SBInstanceCache& instance) 
     {
         // Pull out construction-stage information from the tree.
         const int nBodies = tree.nBodies;
-        const int nDofs   = tree.nDOFs;     // this is the number of u's (nu)
-        const int nSqDofs = tree.sumSqDOFs; // sum(ndof^2) for each joint
-        const int maxNQs  = tree.maxNQs;    // allocate the max # q's we'll ever need     
 
         presUDotPool.resize(instance.getTotalNumPresUDot());
 
-        mobilizerCentrifugalForces.resize(nBodies);           
-        mobilizerCentrifugalForces[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
-
-        totalCentrifugalForces.resize(nBodies);           
-        totalCentrifugalForces[GroundIndex] = SpatialVec(Vec3(0),Vec3(0));
-
         Y.resize(nBodies); // TODO: op space compliance kernel (see Jain 2011)
         Y[GroundIndex] = SpatialMat(Mat33(0));
     }
@@ -1103,7 +1143,6 @@ public:
 public:
     // udot, qdotdot cache space is provided directly by the State.
 
-
     Vector_<SpatialVec> bodyAccelerationInGround; // nb (A_GB)
 
     // This is where the calculated prescribed motion "taus" go. (That is, 
@@ -1121,7 +1160,7 @@ public:
 
 public:
     void allocate(const SBTopologyCache& topo,
-                  const SBModelCache&    model,
+                  const SBModelCache&,
                   const SBInstanceCache& instance) 
     {
         // Pull out topology-stage information from the tree.
@@ -1426,7 +1465,7 @@ public:
     }
 
     const Vector& getQ() const {
-        assert(stage >= Stage::Position);
+        assert(stage >= Stage::Model);
         assert(q);
         return *q;
     }
@@ -1438,7 +1477,7 @@ public:
     }
 
     const Vector& getU() const {
-        assert(stage >= Stage::Velocity);
+        assert(stage >= Stage::Model);
         assert(u);
         return *u;
     }
@@ -1501,7 +1540,7 @@ public:
 
     // Position
     Vector& updQErr() const {
-        assert(stage == Stage::Position);
+        assert(stage > Stage::Instance);
         assert(qErr);
         return *qErr;
     }
@@ -1516,7 +1555,7 @@ public:
         return *tpc;
     }
     const SBTreePositionCache& getTreePositionCache() const {
-        assert(stage > Stage::Position);
+        assert(stage > Stage::Instance);
         assert(tpc);
         return *tpc;
     }
@@ -1533,7 +1572,7 @@ public:
 
     // Velocity
     Vector& updQDot() const {
-        assert(stage == Stage::Velocity);
+        assert(stage > Stage::Instance);
         assert(qdot);
         return *qdot;
     }
@@ -1543,7 +1582,7 @@ public:
         return *qdot;
     }
     Vector& updUErr() const {
-        assert(stage == Stage::Velocity);
+        assert(stage > Stage::Instance);
         assert(uErr);
         return *uErr;
     }
@@ -1558,7 +1597,7 @@ public:
         return *tvc;
     }
     const SBTreeVelocityCache& getTreeVelocityCache() const {
-        assert(stage > Stage::Velocity);
+        assert(stage > Stage::Instance);
         assert(tvc);
         return *tvc;
     }
