@@ -449,26 +449,49 @@ void testByComparingToSDFASTWithConstraint() {
     Inertia inertia = Inertia(Mat33(0.1, 0.01, 0.01,
                                     0.01, 0.1, 0.01,
                                     0.01, 0.01, 0.1));
-    MobilizedBody::Gimbal body1(matter.updGround(), MassProperties(10.0, Vec3(0), inertia));
-    MobilizedBody::Gimbal body2(body1, Vec3(0, -0.1, 0.2), MassProperties(20.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
-    MobilizedBody::Gimbal body3(body1, Vec3(0, -0.1, -0.2), MassProperties(20.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
-    MobilizedBody::Gimbal body4(body2, Vec3(0, -0.2, 0), MassProperties(30.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
-    MobilizedBody::Gimbal body5(body3, Vec3(0, -0.2, 0), MassProperties(30.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
+    MobilizedBody::Gimbal body1(matter.updGround(), 
+        MassProperties(10.0, Vec3(0), inertia));
+    MobilizedBody::Gimbal body2(body1, Vec3(0, -0.1, 0.2), 
+        MassProperties(20.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
+    MobilizedBody::Gimbal body3(body1, Vec3(0, -0.1, -0.2), 
+        MassProperties(20.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
+    MobilizedBody::Gimbal body4(body2, Vec3(0, -0.2, 0), 
+        MassProperties(30.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
+    MobilizedBody::Gimbal body5(body3, Vec3(0, -0.2, 0), 
+        MassProperties(30.0, Vec3(0), inertia), Vec3(0, 0.2, 0));
     Constraint::Rod constraint(body4, body5, 0.15);
     State state = system.realizeTopology();
     system.realize(state, Stage::Velocity);
-    system.project(state, 1e-10);
+    // After I changed the Rod constraint to use distance rather than distance^2
+    // it assembled slightly differently and then wouldn't pass this test to
+    // this precision. So I replaced the project() call with the result from
+    // the older squared equations. The result I was getting from project() with
+    // the distance equations was:
+    //          0 0 0 0.188972205696439 0 0 -0.188972205696439 0 0 
+    //          0.062986198663389 0 0 -0.062986198663389 0 0
+    // which is nearly identical to those below, but different enough to cause 
+    // a failure here. With the same set of q's, the reactions should be the 
+    // same regardless of the equations being used for Rod. (sherm 140506)
+    //system.project(state, 1e-10);
+    Real q[15]={0,0,0,0.189000969332574,0,0,-0.189000969332574,0,0,
+                0.0628990902570866,0,0,-0.0628990902570866,0,0};
+    state.updQ() = Vector(15, q);
     system.realize(state, Stage::Acceleration);
     
     // Calculate reaction forces, and compare to the values that were generated by SD/FAST.
     
     Vector_<SpatialVec> reaction(matter.getNumBodies());
     matter.calcMobilizerReactionForces(state, reaction);
-    assertEqual(~body1.getBodyTransform(state).R()*reaction[body1.getMobilizedBodyIndex()], SpatialVec(Vec3(0, 0, 0), Vec3(-0.000626, 1077.988912, 0.000030)), 1e-5);
-    assertEqual(~body2.getBodyTransform(state).R()*reaction[body2.getMobilizedBodyIndex()], SpatialVec(Vec3(0, 0, 0), Vec3(-0.005038, 495.288692, -18.767467)), 1e-5);
-    assertEqual(~body3.getBodyTransform(state).R()*reaction[body3.getMobilizedBodyIndex()], SpatialVec(Vec3(0, 0, 0), Vec3(0.004236, 495.287857, 18.767535)), 1e-5);
-    assertEqual(~body4.getBodyTransform(state).R()*reaction[body4.getMobilizedBodyIndex()], SpatialVec(Vec3(0, 0, 0), Vec3(0.006251, 303.365940, -0.202330)), 1e-5);
-    assertEqual(~body5.getBodyTransform(state).R()*reaction[body5.getMobilizedBodyIndex()], SpatialVec(Vec3(0, 0, 0), Vec3(-0.005933, 303.365472, 0.202301)), 1e-5);
+    assertEqual(~body1.getBodyTransform(state).R()*reaction[body1.getMobilizedBodyIndex()], 
+                SpatialVec(Vec3(0, 0, 0), Vec3(-0.000626, 1077.988912, 0.000030)), 1e-5);
+    assertEqual(~body2.getBodyTransform(state).R()*reaction[body2.getMobilizedBodyIndex()], 
+                SpatialVec(Vec3(0, 0, 0), Vec3(-0.005038, 495.288692, -18.767467)), 1e-5);
+    assertEqual(~body3.getBodyTransform(state).R()*reaction[body3.getMobilizedBodyIndex()], 
+                SpatialVec(Vec3(0, 0, 0), Vec3(0.004236, 495.287857, 18.767535)), 1e-5);
+    assertEqual(~body4.getBodyTransform(state).R()*reaction[body4.getMobilizedBodyIndex()], 
+                SpatialVec(Vec3(0, 0, 0), Vec3(0.006251, 303.365940, -0.202330)), 1e-5);
+    assertEqual(~body5.getBodyTransform(state).R()*reaction[body5.getMobilizedBodyIndex()], 
+                SpatialVec(Vec3(0, 0, 0), Vec3(-0.005933, 303.365472, 0.202301)), 1e-5);
     
     // Now set it to a different configuration and try again.
     
